@@ -12,9 +12,9 @@ into the package namespace:
 
 """
 
-__author__ = '`the Bloom lab <https://research.fhcrc.org/bloom/en.html>`_'
-__email__ = 'jbloom@fredhutch.org'
-__version__ = '0.4'
-__url__ = 'https://github.com/jbloomlab/binarymap'
+__author__ = "`the Bloom lab <https://research.fhcrc.org/bloom/en.html>`_"
+__email__ = "jbloom@fredhutch.org"
+__version__ = "0.5"
+__url__ = "https://github.com/jbloomlab/binarymap"
 
 from binarymap.binarymap import BinaryMap  # noqa: F401
